#!/usr/bin/perl

# WODM Integration installer

use Getopt::Long;
use Cwd;
use File::Find;
use File::Copy;#::Recursive qw(dircopy);

our $TOP = '';

GetOptions("TOP=s" => \$TOP);

if ($TOP eq '')
{
    print "Usage: $0 --TOP=<Top directory of MDMCS installation>";
    print "\n";
    exit;
}
my $pwd = cwd();

my $source_dir = Cwd::realpath('..');
my $mdmcs_resources_dir = $TOP."/locales";
my $mdmcs_installation_dir = $TOP."/etc/default/wodm";
my $custom_tools_installation_dir = $TOP."/public_html/user";

mkdir $mdmcs_installation_dir;
mkdir $mdmcs_installation_dir."/jars";
mkdir $mdmcs_installation_dir."/jars/eclipse_plugins";
mkdir $mdmcs_installation_dir."/config";

mkdir $custom_tools_installation_dir."/wodm";
mkdir $custom_tools_installation_dir."/wodm/css";
mkdir $custom_tools_installation_dir."/wodm/images";
mkdir $custom_tools_installation_dir."/wodm/js";
mkdir $custom_tools_installation_dir."/wodm/jsp";
mkdir $custom_tools_installation_dir."/wodm/css/decisions";
mkdir $custom_tools_installation_dir."/wodm/js/decisions";
mkdir $custom_tools_installation_dir."/wodm/jsp/decisions";
mkdir $custom_tools_installation_dir."/wodm/js/decisions/nls";
mkdir $custom_tools_installation_dir."/wodm/js/decisions/nls/de-de";
mkdir $custom_tools_installation_dir."/wodm/js/decisions/nls/es-es";
mkdir $custom_tools_installation_dir."/wodm/js/decisions/nls/fr-fr";
mkdir $custom_tools_installation_dir."/wodm/js/decisions/nls/it-it";
mkdir $custom_tools_installation_dir."/wodm/js/decisions/nls/ja-jp";
mkdir $custom_tools_installation_dir."/wodm/js/decisions/nls/ko-kr";
mkdir $custom_tools_installation_dir."/wodm/js/decisions/nls/pt-br";
mkdir $custom_tools_installation_dir."/wodm/js/decisions/nls/ru-ru";
mkdir $custom_tools_installation_dir."/wodm/js/decisions/nls/zh-cn";
mkdir $custom_tools_installation_dir."/wodm/js/decisions/nls/zh-tw";
mkdir $custom_tools_installation_dir."/wodm/js/decisions/templates";

copy ($source_dir."/jars/wodm.command.jar", $mdmcs_installation_dir."/jars/wodm.command.jar");
copy ($source_dir."/jars/wodm.extensions.jar", $mdmcs_installation_dir."/jars/wodm.extensions.jar");
copy ($source_dir."/jars/wodmBranchHelper.jar", $mdmcs_installation_dir."/jars/wodmBranchHelper.jar");

# Update of data_entry_properties.xml has to be done by hand.
#copy ($source_dir."/customtools/etc/default/data_entry_properties.xml",
#      $TOP."/etc/default/data_entry_properties.xml");

# Update flow-config.xml (using unique tag in base file).
my $tagText = "<!-- Extensions entries added here -->";
my $wodmText = $tagText."\n"
."<!-- Start of WODM additions -->\n"
."<async-flow path=\"wodmLogin\" command=\"com.ibm.ccd.solution.wodm.WODMCustomTabCommand\" method=\"login\"/>\n"
."<async-flow path=\"wodmGetDecisionsAndRules\" command=\"com.ibm.ccd.solution.wodm.WODMCustomTabCommand\" method=\"getDecisionTypesAndRules\"/>\n"
."<async-flow path=\"wodmViewRule\" command=\"com.ibm.ccd.solution.wodm.WODMCustomTabCommand\" method=\"viewRule\"/>\n"
."<async-flow path=\"wodmGetRuleForEditing\" command=\"com.ibm.ccd.solution.wodm.WODMCustomTabCommand\" method=\"getRuleForEditing\"/>\n"
."<async-flow path=\"wodmSaveRule\" command=\"com.ibm.ccd.solution.wodm.WODMCustomTabCommand\" method=\"saveRule\"/>\n"
."<async-flow path=\"wodmCancelRuleEditing\" command=\"com.ibm.ccd.solution.wodm.WODMCustomTabCommand\" method=\"cancelRuleEditing\"/>\n"
."<async-flow path=\"wodmDeleteRule\" command=\"com.ibm.ccd.solution.wodm.WODMCustomTabCommand\" method=\"deleteRule\"/>\n"
."<async-flow path=\"wodmGetRuleTemplates\" command=\"com.ibm.ccd.solution.wodm.WODMCustomTabCommand\" method=\"getRuleTemplates\"/>\n"
."<async-flow path=\"wodmCreateRule\" command=\"com.ibm.ccd.solution.wodm.WODMCustomTabCommand\" method=\"createRule\"/>\n"
."<!-- End of WODM additions -->\n";

my $flow_old = $TOP."/etc/default/flow-config.old";
my $flow_new = $TOP."/etc/default/flow-config.xml";
copy($flow_new, $flow_old);

my $oldEoR = $/;
$/ = undef;
open OLD, "< $flow_old" or die "Can't read input file ".$flow_old.": $!\n";
my $template = <OLD>;
close OLD;
$/ = $oldEoR;

# Substitute WODM additions text just after "<!-- Extensions entries added here -->"
# Do this only if it has not been done before.
if (!($template =~ /WODM/)) {
    $template =~ s|$tagText|$wodmText|;

    open NEW, "> $flow_new" or print "Can't open file ".$flow_new." for write: $!\n";
    print NEW $template;
    close NEW;
}
unlink("$flow_old");

# Install the WODM config file.
copy ($source_dir."/customtools/wodm-config.xml",
      $mdmcs_installation_dir."/config/wodm-config.xml");

# Install the WODM classpath file.
copy ($source_dir."/customtools/wodm-classpath.txt",
      $mdmcs_installation_dir."/config/wodm-classpath.txt");

# All these target directories should already exist in the product installation.
#
copy ($source_dir."/customtools/locales/de_DE/MDMCS_WODM_Messages.xml",
      $mdmcs_resources_dir."/de_DE/custom_resource_bundle/MDMCS_WODM_Messages.xml");
copy ($source_dir."/customtools/locales/en_US/MDMCS_WODM_Messages.xml",
      $mdmcs_resources_dir."/en_US/custom_resource_bundle/MDMCS_WODM_Messages.xml");
copy ($source_dir."/customtools/locales/fr_FR/MDMCS_WODM_Messages.xml",
      $mdmcs_resources_dir."/fr_FR/custom_resource_bundle/MDMCS_WODM_Messages.xml");
copy ($source_dir."/customtools/locales/it_IT/MDMCS_WODM_Messages.xml",
      $mdmcs_resources_dir."/it_IT/custom_resource_bundle/MDMCS_WODM_Messages.xml");
copy ($source_dir."/customtools/locales/ja_JP/MDMCS_WODM_Messages.xml",
      $mdmcs_resources_dir."/ja_JP/custom_resource_bundle/MDMCS_WODM_Messages.xml");
copy ($source_dir."/customtools/locales/ko_KR/MDMCS_WODM_Messages.xml",
      $mdmcs_resources_dir."/ko_KR/custom_resource_bundle/MDMCS_WODM_Messages.xml");
copy ($source_dir."/customtools/locales/pt_BR/MDMCS_WODM_Messages.xml",
      $mdmcs_resources_dir."/pt_BR/custom_resource_bundle/MDMCS_WODM_Messages.xml");
copy ($source_dir."/customtools/locales/ru_RU/MDMCS_WODM_Messages.xml",
      $mdmcs_resources_dir."/ru_RU/custom_resource_bundle/MDMCS_WODM_Messages.xml");
copy ($source_dir."/customtools/locales/zh_CN/MDMCS_WODM_Messages.xml",
      $mdmcs_resources_dir."/zh_CN/custom_resource_bundle/MDMCS_WODM_Messages.xml");
copy ($source_dir."/customtools/locales/zh_TW/MDMCS_WODM_Messages.xml",
      $mdmcs_resources_dir."/zh_TW/custom_resource_bundle/MDMCS_WODM_Messages.xml");

copy ($source_dir."/customtools/user/wodm/css/decisions/webcomponents.css",
      $custom_tools_installation_dir."/wodm/css/decisions/webcomponents.css");
copy ($source_dir."/customtools/user/wodm/css/decisions/table.css",
      $custom_tools_installation_dir."/wodm/css/decisions/table.css");
copy ($source_dir."/customtools/user/wodm/css/decisions/syntacticeditor.css",
      $custom_tools_installation_dir."/wodm/css/decisions/syntacticeditor.css");
copy ($source_dir."/customtools/user/wodm/css/decisions/manageDecisions.css",
      $custom_tools_installation_dir."/wodm/css/decisions/manageDecisions.css");
copy ($source_dir."/customtools/user/wodm/css/decisions/loginForm.css",
      $custom_tools_installation_dir."/wodm/css/decisions/loginForm.css");

copy ($source_dir."/customtools/user/wodm/images/checkmark.gif",
      $custom_tools_installation_dir."/wodm/images/checkmark.gif");
copy ($source_dir."/customtools/user/wodm/images/error.gif",
      $custom_tools_installation_dir."/wodm/images/error.gif");
copy ($source_dir."/customtools/user/wodm/images/loading_large.gif",
      $custom_tools_installation_dir."/wodm/images/loading_large.gif");
copy ($source_dir."/customtools/user/wodm/images/titlepaneTitleRepeatBlueNew.gif",
      $custom_tools_installation_dir."/wodm/images/titlepaneTitleRepeatBlueNew.gif");

copy ($source_dir."/customtools/user/wodm/js/decisions/manageDecisionsController.js",
      $custom_tools_installation_dir."/wodm/js/decisions/manageDecisionsController.js");
copy ($source_dir."/customtools/user/wodm/js/decisions/LoginFormDialog.js",
      $custom_tools_installation_dir."/wodm/js/decisions/LoginFormDialog.js");
copy ($source_dir."/customtools/user/wodm/js/decisions/LoginForm.js",
      $custom_tools_installation_dir."/wodm/js/decisions/LoginForm.js");
copy ($source_dir."/customtools/user/wodm/js/decisions/ErrorWidget.js",
      $custom_tools_installation_dir."/wodm/js/decisions/ErrorWidget.js");
copy ($source_dir."/customtools/user/wodm/js/decisions/EditRuleDialog.js",
      $custom_tools_installation_dir."/wodm/js/decisions/EditRuleDialog.js");
copy ($source_dir."/customtools/user/wodm/js/decisions/AddRuleDialog.js",
      $custom_tools_installation_dir."/wodm/js/decisions/AddRuleDialog.js");
copy ($source_dir."/customtools/user/wodm/js/decisions/nls/manageDecisionsController.js",
      $custom_tools_installation_dir."/wodm/js/decisions/nls/manageDecisionsController.js");
copy ($source_dir."/customtools/user/wodm/js/decisions/nls/LoginForm.js",
      $custom_tools_installation_dir."/wodm/js/decisions/nls/LoginForm.js");
copy ($source_dir."/customtools/user/wodm/js/decisions/nls/AddRuleDialog.js",
      $custom_tools_installation_dir."/wodm/js/decisions/nls/AddRuleDialog.js");
copy ($source_dir."/customtools/user/wodm/js/decisions/nls/de-de/manageDecisionsController.js",
      $custom_tools_installation_dir."/wodm/js/decisions/nls/de-de/manageDecisionsController.js");
copy ($source_dir."/customtools/user/wodm/js/decisions/nls/es-es/manageDecisionsController.js",
      $custom_tools_installation_dir."/wodm/js/decisions/nls/es-es/manageDecisionsController.js");
copy ($source_dir."/customtools/user/wodm/js/decisions/nls/fr-fr/manageDecisionsController.js",
      $custom_tools_installation_dir."/wodm/js/decisions/nls/fr-fr/manageDecisionsController.js");
copy ($source_dir."/customtools/user/wodm/js/decisions/nls/it-it/manageDecisionsController.js",
      $custom_tools_installation_dir."/wodm/js/decisions/nls/it-it/manageDecisionsController.js");
copy ($source_dir."/customtools/user/wodm/js/decisions/nls/ja-jp/manageDecisionsController.js",
      $custom_tools_installation_dir."/wodm/js/decisions/nls/ja-jp/manageDecisionsController.js");
copy ($source_dir."/customtools/user/wodm/js/decisions/nls/ko-kr/manageDecisionsController.js",
      $custom_tools_installation_dir."/wodm/js/decisions/nls/ko-kr/manageDecisionsController.js");
copy ($source_dir."/customtools/user/wodm/js/decisions/nls/pt-br/manageDecisionsController.js",
      $custom_tools_installation_dir."/wodm/js/decisions/nls/pt-br/manageDecisionsController.js");
copy ($source_dir."/customtools/user/wodm/js/decisions/nls/ru-ru/manageDecisionsController.js",
      $custom_tools_installation_dir."/wodm/js/decisions/nls/ru-ru/manageDecisionsController.js");
copy ($source_dir."/customtools/user/wodm/js/decisions/nls/zh-cn/manageDecisionsController.js",
      $custom_tools_installation_dir."/wodm/js/decisions/nls/zh-cn/manageDecisionsController.js");
copy ($source_dir."/customtools/user/wodm/js/decisions/nls/zh-tw/manageDecisionsController.js",
      $custom_tools_installation_dir."/wodm/js/decisions/nls/zh-tw/manageDecisionsController.js");
copy ($source_dir."/customtools/user/wodm/js/decisions/nls/de-de/AddRuleDialog.js",
      $custom_tools_installation_dir."/wodm/js/decisions/nls/de-de/AddRuleDialog.js");
copy ($source_dir."/customtools/user/wodm/js/decisions/nls/es-es/AddRuleDialog.js",
      $custom_tools_installation_dir."/wodm/js/decisions/nls/es-es/AddRuleDialog.js");
copy ($source_dir."/customtools/user/wodm/js/decisions/nls/fr-fr/AddRuleDialog.js",
      $custom_tools_installation_dir."/wodm/js/decisions/nls/fr-fr/AddRuleDialog.js");
copy ($source_dir."/customtools/user/wodm/js/decisions/nls/it-it/AddRuleDialog.js",
      $custom_tools_installation_dir."/wodm/js/decisions/nls/it-it/AddRuleDialog.js");
copy ($source_dir."/customtools/user/wodm/js/decisions/nls/ja-jp/AddRuleDialog.js",
      $custom_tools_installation_dir."/wodm/js/decisions/nls/ja-jp/AddRuleDialog.js");
copy ($source_dir."/customtools/user/wodm/js/decisions/nls/ko-kr/AddRuleDialog.js",
      $custom_tools_installation_dir."/wodm/js/decisions/nls/ko-kr/AddRuleDialog.js");
copy ($source_dir."/customtools/user/wodm/js/decisions/nls/pt-br/AddRuleDialog.js",
      $custom_tools_installation_dir."/wodm/js/decisions/nls/pt-br/AddRuleDialog.js");
copy ($source_dir."/customtools/user/wodm/js/decisions/nls/ru-ru/AddRuleDialog.js",
      $custom_tools_installation_dir."/wodm/js/decisions/nls/ru-ru/AddRuleDialog.js");
copy ($source_dir."/customtools/user/wodm/js/decisions/nls/zh-cn/AddRuleDialog.js",
      $custom_tools_installation_dir."/wodm/js/decisions/nls/zh-cn/AddRuleDialog.js");
copy ($source_dir."/customtools/user/wodm/js/decisions/nls/zh-tw/AddRuleDialog.js",
      $custom_tools_installation_dir."/wodm/js/decisions/nls/zh-tw/AddRuleDialog.js");
copy ($source_dir."/customtools/user/wodm/js/decisions/nls/de-de/LoginForm.js",
      $custom_tools_installation_dir."/wodm/js/decisions/nls/de-de/LoginForm.js");
copy ($source_dir."/customtools/user/wodm/js/decisions/nls/es-es/LoginForm.js",
      $custom_tools_installation_dir."/wodm/js/decisions/nls/es-es/LoginForm.js");
copy ($source_dir."/customtools/user/wodm/js/decisions/nls/fr-fr/LoginForm.js",
      $custom_tools_installation_dir."/wodm/js/decisions/nls/fr-fr/LoginForm.js");
copy ($source_dir."/customtools/user/wodm/js/decisions/nls/it-it/LoginForm.js",
      $custom_tools_installation_dir."/wodm/js/decisions/nls/it-it/LoginForm.js");
copy ($source_dir."/customtools/user/wodm/js/decisions/nls/ja-jp/LoginForm.js",
      $custom_tools_installation_dir."/wodm/js/decisions/nls/ja-jp/LoginForm.js");
copy ($source_dir."/customtools/user/wodm/js/decisions/nls/ko-kr/LoginForm.js",
      $custom_tools_installation_dir."/wodm/js/decisions/nls/ko-kr/LoginForm.js");
copy ($source_dir."/customtools/user/wodm/js/decisions/nls/pt-br/LoginForm.js",
      $custom_tools_installation_dir."/wodm/js/decisions/nls/pt-br/LoginForm.js");
copy ($source_dir."/customtools/user/wodm/js/decisions/nls/ru-ru/LoginForm.js",
      $custom_tools_installation_dir."/wodm/js/decisions/nls/ru-ru/LoginForm.js");
copy ($source_dir."/customtools/user/wodm/js/decisions/nls/zh-cn/LoginForm.js",
      $custom_tools_installation_dir."/wodm/js/decisions/nls/zh-cn/LoginForm.js");
copy ($source_dir."/customtools/user/wodm/js/decisions/nls/zh-tw/LoginForm.js",
      $custom_tools_installation_dir."/wodm/js/decisions/nls/zh-tw/LoginForm.js");

copy ($source_dir."/customtools/user/wodm/js/decisions/templates/AddRuleDialogTemplate.html",
      $custom_tools_installation_dir."/wodm/js/decisions/templates/AddRuleDialogTemplate.html");
copy ($source_dir."/customtools/user/wodm/js/decisions/templates/loginFormTemplate.html",
      $custom_tools_installation_dir."/wodm/js/decisions/templates/loginFormTemplate.html");
copy ($source_dir."/customtools/user/wodm/js/decisions/templates/ErrorWidget.html",
      $custom_tools_installation_dir."/wodm/js/decisions/templates/ErrorWidget.html");

copy ($source_dir."/customtools/user/wodm/jsp/wodmHeader.jsp.include",
      $custom_tools_installation_dir."/wodm/jsp/wodmHeader.jsp.include");
copy ($source_dir."/customtools/user/wodm/jsp/decisions/editRule.jsp",
      $custom_tools_installation_dir."/wodm/jsp/decisions/editRule.jsp");
copy ($source_dir."/customtools/user/wodm/jsp/decisions/viewRule.jsp",
      $custom_tools_installation_dir."/wodm/jsp/decisions/viewRule.jsp");
copy ($source_dir."/customtools/user/wodm/jsp/decisions/manageDecisions.jsp",
      $custom_tools_installation_dir."/wodm/jsp/decisions/manageDecisions.jsp");

my $jarfilepath = $TOP."/bin/conf/classpath/jars-custom.txt";

# Append jars to the end of jars-custom.txt

my $customjarpath =
    "\n"."etc/default/wodm/jars/wodm.command.jar";

open FILE, ">>$jarfilepath";
print FILE $customjarpath, "\n";
close FILE;

# Import asset data model to specified company in MDMCE
# TODO!!

exit;
