/*******************************************************************************
 * {COPYRIGHT-TOP} *** * IBM Confidential * OCO Source Materials 5725-E59
 * 
 * (c) Copyright IBM Corp. 2012 All Rights Reserved.
 * 
 * The source code for this program is not published or otherwise divested of
 * its trade secrets, irrespective of what has been deposited with the U.S.
 * Copyright Office.
 ******************************************************************************/
define(
        [ "dojo/_base/declare", "dijit/_Widget", "dijit/_Templated",
                "dojo/text!./templates/AddRuleDialogTemplate.html",
                "dojo/i18n!wodm/decisions/nls/AddRuleDialog", "dojo/on",
                "dojo/_base/xhr", "dojo/_base/lang", "dojo/_base/array",
                "dojo/dom-construct", "dojo/dom-class",
                "ibm/ccd/dialogs/CommonDialog", "dijit/form/FilteringSelect",
                "dojo/data/ItemFileReadStore", "dijit/form/Button",
                "wodm/decisions/ErrorWidget", "dijit/form/TextBox" ],
        function(declare, _Widget, _Templated, template, i18nResources, on,
                xhr, lang, baseArray, domConstruct, domClass, CommonDialog,
                FilteringSelect, ItemFileReadStore, Button, ErrorWidget) {

            /**
             * The module wodm.decisions.AddRuleDialog provides a Widget that
             * allows the user to add new rule to a decision type in a Dialog.
             */

            var AddRuleDialog = declare(
                    "wodm.decisions.AddRuleDialog",
                    [ _Widget, _Templated ],
                    {
                        templateString : template,
                        widgetsInTemplate : true,

                        /**
                         * WODM common configuration properties.
                         */
                        _config : dojo.config.wodm,

                        baseClass : "wodmDecisionsAddRuleDialog",

                        /**
                         * URLS for various xhr requests/navigation links for
                         * the WODM Add Rule Dialog Widget.
                         */
                        _urls : {

                            GET_RULE_TEMPLATES : "wodmGetRuleTemplates.wpc",
                            CREATE_RULE : "wodmCreateRule.wpc"

                        },

                        /**
                         * Object containing information for the decision type
                         * to which the new rule needs to be added. This object
                         * should contain the following: decisionType: {id of
                         * the decision type} decisionTypeName: {display name of
                         * the decision type} ruleProjectName: {name of the rule
                         * project associated with the decision type}
                         */
                        decisionTypeInfo : null,

                        /**
                         * Object containing properties for buttons that will be
                         * added to the Add Rule Dialog button bar.
                         */
                        _buttonPropsMap : null,

                        /**
                         * Reference to manageDecisionsController, for login handling.
                         */
                        controller : null,

                        postMixInProperties : function() {

                            this.inherited(arguments);
                            this._i18nResources = i18nResources;

                        },

                        postCreate : function() {

                            this.inherited(arguments);
                            this._createRuleTemplateSelect();
                            this._initializeButtonPropsMap();

                        },

                        _errorNode : null,

                        /**
                         * Object containing reference to the underlying
                         * CommonDialog widget.
                         */
                        _commonDialog : null,

                        /**
                         * Object containing reference to the FilteringSelect
                         * Widget used for selecting a rule template.
                         */
                        _templateSelect : null,

                        /**
                         * Store used for the _templateSelect FilteringSelect.
                         */
                        _templateSelectStore : null,

                        /**
                         * Array of connection handles. Used to store dojo
                         * connections that can be disconnected on destroy.
                         */
                        _connectionHandles : [],

                        /**
                         * Returns a url which is used for getting the rule
                         * templates associated with the rule project for the
                         * current decision type.
                         */
                        _getTemplateSelectStoreUrl : function() {

                            return this._urls.GET_RULE_TEMPLATES
                                    + "?ruleProjectName="
                                    + this.decisionTypeInfo.ruleProjectName
                                    + "&containerType="
                                    + this._config.containerType;
                        },

                        /**
                         * Creates the FilteringSelect for the Rule Template
                         * Field in the AddRuleDialog Widget.
                         */
                        _createRuleTemplateSelect : function() {

                            var store, select;
                            store = new ItemFileReadStore({

                                url : this._getTemplateSelectStoreUrl(),

                                clearOnClose : true

                            });
                            select = new FilteringSelect(
                                    {
                                        name : "ruleTemplate",
                                        store : store,
                                        placeHolder : this._i18nResources.ADD_RULE_DIALOG_TEMPLATE_SELECT_DEFAULT_TEXT,
                                        required : true

                                    }, this._templateFieldPlaceHolder);
                            select.startup();
                            this._templateSelect = select;
                            this._templateSelectStore = store;

                        },

                        /**
                         * Creates an instance of the CommonDialog widget.
                         */
                        _createDialog : function() {

                            this._commonDialog = new CommonDialog(
                                    {

                                        title : this._i18nResources.ADD_RULE_DIALOG_TITLE,
                                        showButtonBar : true,
                                        showOkButton : false,
                                        showCancelButton : true,
                                        style : "width: 350px;"

                                    });

                            this._commonDialog.contentNode
                                    .appendChild(this.domNode);
                            this.createDialogButtons();

                        },

                        /**
                         * Initializes the properties of different buttons used
                         * in the Button Bar for the Add Rule Dialog. This
                         * includes definition as well as handlers for each
                         * button. The properties will be available in the
                         * _buttonPropsMap after initialization.
                         */
                        _initializeButtonPropsMap : function() {

                            this._buttonPropsMap = {

                                "wodmAddRuleDialog.save" : {

                                    definition : {

                                        id : "wodmAddRuleDialog.save",
                                        label : this._i18nResources.ADD_RULE_DIALOG_SAVE_BUTTON_LABEL,
                                        title : this._i18nResources.ADD_RULE_DIALOG_SAVE_BUTTON_LABEL
                                    },
                                    handlers : {

                                        click : lang.hitch(this, this._onSave,
                                                false)

                                    }

                                },

                                "wodmAddRuleDialog.saveAndEdit" : {

                                    definition : {

                                        id : "wodmAddRuleDialog.saveAndEdit",
                                        label : this._i18nResources.ADD_RULE_DIALOG_SAVE_AND_EDIT_BUTTON_LABEL,
                                        title : this._i18nResources.ADD_RULE_DIALOG_SAVE_AND_EDIT_BUTTON_LABEL
                                    },
                                    handlers : {

                                        click : lang.hitch(this, this._onSave,
                                                true)

                                    }

                                }

                            }

                        },

                        /**
                         * Creates Button widgets for the Button Bar in the Add
                         * Rule Dialog Widget.
                         */
                        createDialogButtons : function() {

                            this
                                    ._addButton(this._buttonPropsMap["wodmAddRuleDialog.save"]);
                            this
                                    ._addButton(this._buttonPropsMap["wodmAddRuleDialog.saveAndEdit"]);

                        },

                        /**
                         * Creates a new dijit.form.Button object based on the
                         * given properties object and adds it to the dialog
                         * button bar.
                         * 
                         * @param buttonProps
                         *            {Object} an object containing properties
                         *            of the button to be created. This includes
                         *            definition as well as handlers. *
                         */
                        _addButton : function(buttonProps) {

                            var button, connectionHandle;
                            button = new Button(buttonProps.definition);
                            connectionHandle = on(button, "click",
                                    buttonProps.handlers.click);
                            this._connectionHandles.push(connectionHandle);
                            this._commonDialog.buttonBar.insertBefore(
                                    button.domNode,
                                    this._commonDialog.cancelButton.domNode);

                        },

                        /**
                         * Displays the AddRuleDialog Widget.
                         */
                        show : function() {

                            if (!this._commonDialog) {
                                this._createDialog();
                            } else {

                                this.ruleNameTxtBox.set("value", "");
                                this._destroyErrorWidget();

                               if(this._templateSelect){
                                 this._templateSelect.reset();
                               }
                            }

                            this._commonDialog.show();

                        },

                        /**
                         * Hides the AddRuleDialog Widget.
                         */
                        hide : function() {
                            this._commonDialog.hide();
                        },

                        _destroyErrorWidget : function() {
                            if (this._errorWidget) {
                                domConstruct.destroy(this._errorWidget.domNode);
                                this._errorWidget = null;
                            }

                        },

                        /**
                         * Hook for the set method to work for the
                         * decisionTypeInfo property.
                         * 
                         * If the decisionTypeInfo attribute is changed, the
                         * template filtering select is updated as well as the
                         * decision type name field.
                         * 
                         * 
                         * @param value
                         *            {Object} object containing information
                         *            about the decision type to which the new
                         *            rule needs to be added to.
                         */
                        _setDecisionTypeInfoAttr : function(value) {

                            if (this.decisionTypeInfo
                                    && this.decisionTypeInfo.decisionType !== value.decisionType
                                    && this._templateSelectStore !== null) {

                                this.decisionTypeInfo = value;
                                this._decisionTypeField.innerText = this.decisionTypeInfo.decisionTypeName;
                                this._templateSelectStore.close();
                                this._templateSelectStore.url = this
                                        ._getTemplateSelectStoreUrl();
                                this._templateSelect.store = this._templateSelectStore;
                                this._templateSelect.reset();

                            } else {

                                this.decisionTypeInfo = value;
                                this._decisionTypeField.innerText = this.decisionTypeInfo.decisionTypeName;
                            }

                        },

                        /**
                         * Event handler for the onclick event for the Save and
                         * Save and Edit buttons in the Dialog Button Bar.
                         * 
                         * @param editImmediately
                         *            {boolean} true if 'Save and Edit' was
                         *            clicked, false otherwise.
                         */
                        _onSave : function(editImmediately) {

                            var ruleName, ruleProjectName, ruleTemplateName, contentObj;
                            ruleName = this.ruleNameTxtBox.get("value");
                            ruleTemplateName = this._templateSelect
                                    .get("value");
                            if (!this._validate(ruleName, ruleTemplateName)) {
                                return;
                            }

                            if (!this.controller._verifyAccess(false)) {
                                this.controller._createLoginForm(true, lang.hitch(this,
                                    this._sendSaveNewRuleRequest, ruleName,
                                                                  ruleTemplateName, editImmediately));
                            } else {
                                this._sendSaveNewRuleRequest(ruleName,
                                                             ruleTemplateName, editImmediately);
                            }

                            

                        },

                        /**
                         * Removes any currently displayed errors and displays
                         * the newly provided errors.
                         */
                        _displayErrors : function(errors) {

                            this._destroyErrorWidget();
                            var errorWidget = new ErrorWidget({
                                errors : errors
                            });
                            this._errorWidget = errorWidget;
                            this._errorWidgetDiv
                                    .appendChild(errorWidget.domNode);

                        },

                        /**
                         * Returns true if the given parameters are valid, false
                         * otherwise. If any of the parameters is not valid,
                         * this method will display validation errors and return
                         * false.
                         * 
                         * 
                         * @param ruleName
                         *            {String} name of the rule to create
                         * @param ruleTemplateName
                         *            {String} template to use for creating the
                         *            new rule
                         */
                        _validate : function(ruleName, ruleTemplateName) {

                            var errors = [];
                            if (!ruleName) {
                                errors
                                        .push(this._i18nResources.ADD_RULE_DIALOG_RULE_NAME_VALIDATION_ERROR);
                            }
                            if (!ruleTemplateName) {
                                errors
                                        .push(this._i18nResources.ADD_RULE_DIALOG_RULE_TEMPLATE_VALIDATION_ERROR);
                            }

                            if (errors.length > 0) {
                                this._displayErrors(errors);
                                return false;
                            }

                            return true;

                        },

                        /**
                         * Sends an xhr request to create a new rule with the
                         * given ruleName using the given ruleTemplateName.
                         * 
                         * @param ruleName
                         *            {String} name of the rule to create
                         * @param ruleTemplateName
                         *            {String} template to use for creating the
                         *            new rule
                         * @param editImmediately
                         *            {boolean} value passed on to the call back
                         *            function on a successful save
                         * 
                         */
                        _sendSaveNewRuleRequest : function(ruleName,
                                ruleTemplateName, editImmediately) {

                            contentObj = lang
                                    .mixin(lang.clone(this.decisionTypeInfo),
                                            this._config);
                            contentObj.ruleName = ruleName;
                            contentObj.templateName = ruleTemplateName;
                            var reqObj = {

                                url : this._urls.CREATE_RULE,
                                handleAs : "json",
                                sync : true,
                                content : contentObj,

                                load : lang.hitch(this, function(response) {

                                    this._handleSaveNewRuleRequestResponse(
                                            response, editImmediately);
                                }),
                                error : lang
                                        .hitch(
                                                this,
                                                function(error) {
                                                    var errorObj = {
                                                        error : error
                                                    }
                                                    this
                                                            ._handleSaveNewRuleRequestResponse(errorObj);
                                                })

                            }

                            setTimeout(function() {

                                xhr.get(reqObj);

                            }, 1);

                        },

                        /**
                         * Handles the response received from the create new
                         * rule request.
                         * 
                         * @param response
                         *            {Object} contains the response of the new
                         *            rule creation request
                         * @param editImmediately
                         *            {boolean} indicates whether the user asked
                         *            to edit newly created rule immediately
                         */
                        _handleSaveNewRuleRequestResponse : function(response,
                                editImmediately) {

                            if (response.error) {

                                this._handleResponseError(response.error);
                                return;
                            }

                            this.onSuccessfulSave(response.rule,
                                    editImmediately);

                        },

                        /**
                         * Handles response errors and displays them at the top
                         * in the Dialog Widget.
                         * 
                         * @param error
                         *            {Object} object containing details about
                         *            any errors that occurred while processing
                         *            the request
                         */
                        _handleResponseError : function(error) {

                            if (error.wodmSessionTimeout) {
                                this._config.isUserLoggedIn = false;
                            }

                            this._displayErrors([ error.errorMessage ]);

                        },

                        /**
                         * Invoked on a successful save of the new rule. Users
                         * of this Widget should connect to this method to
                         * receive notification about a successful save
                         * operation.
                         * 
                         * @param newRule
                         *            {Object} json object representing the
                         *            newly created rule
                         * @param editImmediately
                         *            {boolean} indicates whether the user asked
                         *            to edit the rule immediately after
                         *            creation
                         */
                        onSuccessfulSave : function(newRule, editImmediately) {

                        },

                        /**
                         * Widget life cycle method. This function is called by
                         * the dojo framework when destroy is invoked on a
                         * widget. This method should not be invoked directly,
                         * instead invoke the destroy method.
                         */
                        uninitialize : function() {

                            this._destroyErrorWidget();
                            baseArray.forEach(this._connectionHandles,
                                    function(handle) {

                                        handle.remove();

                                    });
                            // This is a templated widget, the dialog is already detroyed by _WidgetBase.destroyRecursive()
                            //this._commonDialog.destroy();

                        }
                    });
            return AddRuleDialog;

        });